#!/bin/bash

# Script to update the dynamic DNS entries

# define your values for the service SPDNS.de
#
# example:
# spdyn_dns_host="test.spdns.de"
# spdyn_dns_token="abcd-efgh-ijkl"

spdyn_dns_host="your_subdomain.spdns.org"
spdyn_dns_token="abcd-efgh-ijkl"

goip_username="abcdefghij12345"
goip_password="1234567890abcde"
goip_dns_host="your_subdomain.goip.de"

external_ip=""

# some colors
LIGHT_BLUE="\033[44m\033[1m"
GREEN="\033[42m\033[1m"
RED="\033[41m\033[1m"
ORANGE="\033[43m\033[1m"
MAGENTA="\033[45m\033[1m"
COLOR_END="\033[0m"

# log to syslog and standard out
function log() {
  logger "$1"
  echo "$1"
}

# set the IP address for SPDNS.de
function set_spdyn_ip_address() {

  if [ "$spdyn_dns_host" == "" ] || [ "$spdyn_dns_token" == "" ] ;then
    log "Please define your DNS values for the service SPDNS.de in this script !"
    return
  fi

  current_set_ip_address=$(nslookup "$spdyn_dns_host" | grep -A2 "answer:" | grep "Address:" | sed 's|Address:\s*||')
  echo "Current set IP address: $current_set_ip_address"

  if [ "$current_set_ip_address" == "$1" ];then
    echo -e "${LIGHT_BLUE}IP address is already set. Nothing to do.${COLOR_END}"
    return
  fi

  # echo "wget -T 10 --user=\"$spdyn_dns_host\" --password=\"$spdyn_dns_token\" \"https://update.spdyn.de/nic/update?hostname=${spdyn_dns_host}&myip=$1\" -O - --quiet"
  response=`wget -T 10 --user="$spdyn_dns_host" --password="$spdyn_dns_token" "https://update.spdyn.de/nic/update?hostname=${spdyn_dns_host}&myip=$1" -O - --quiet`
  result=$?
  if [ $result -ne 0 ];then
    log "Setting the IP address at SPDNS failed."
    return
  fi
  log "$response"

  # valid response:
  # "good 1.2.3.4"
  echo "$response" | grep -q "^good"
  if [ $? -ne 0 ];then
    log "Response from SPDNS does not contain the word \"good\""
    return
  else
    echo -e "${GREEN}IP address was set at SPDNS.${COLOR_END}"
  fi

}

# set the IP address for goip.de
function set_goip_ip_address() {

  if [ "$goip_dns_host" == "" ] || [ "$goip_username" == "" ] || [ "$goip_password" == "" ] ;then
    log "Please define your DNS values for the service goip.de in this script !"
    return
  fi

  current_set_ip_address=$(nslookup "$goip_dns_host" | grep -A2 "answer:" | grep "Address:" | sed 's|Address:\s*||')
  echo "Current set IP address: $current_set_ip_address"

  if [ "$current_set_ip_address" == "$1" ];then
    echo -e "${LIGHT_BLUE}IP address is already set. Nothing to do.${COLOR_END}"
    return
  fi

  echo "wget -T 10 \"www.goip.de/setip?username=${goip_username}&password=${goip_password}&subdomain=${goip_dns_host}&ip=$1&shortResponse=true\" -O - --quiet"
  response=`wget -T 10 "https://www.goip.de/setip?username=${goip_username}&password=${goip_password}&subdomain=${goip_dns_host}&ip=$1&shortResponse=true" -O - --quiet`
  result=$?
  if [ $result -ne 0 ];then
    log "Setting the IP address at goip failed."
    return
  fi
  log "$response"

  # valid response:
  # " subdomain.goip.de"
  echo "$response" | grep -q "^ $goip_dns_host"
  if [ $? -ne 0 ];then
    log "Response from goip does not contain the subdomain."
    return
  else
    echo -e "${GREEN}IP address was set at goip.${COLOR_END}"
  fi

}

# validate the IP address
function is_valid_ip_address()
{
  local  ip=$1
  local  stat=1

  if [[ $ip =~ ^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$ ]]; then
    OIFS=$IFS
    IFS='.'
    ip=($ip)
    IFS=$OIFS
    [[ ${ip[0]} -le 255 && ${ip[1]} -le 255 \
        && ${ip[2]} -le 255 && ${ip[3]} -le 255 ]]
    stat=$?
  fi
  return $stat
}

# get your own IP address from an external server
function get_own_ip_address() {
  external_ip=`wget -T 5 http://checkip4.spdns.de -O - --quiet`

  # use a second
  if [ $? -ne 0 ];then
    external_ip=`wget -T 5 https://api.ipify.org -O - --quiet`
    if [ $? -ne 0 ];then
      log "Problem to get own IP address."
      exit 1
    fi
  fi

  if ! is_valid_ip_address $external_ip; then
    log "IP address looks bad."
    exit 2
  fi
}

get_own_ip_address

echo "Found the following own IP address: $external_ip"

# set the IP address at SPDNS and goip
set_spdyn_ip_address "$external_ip"
set_goip_ip_address "$external_ip"
